"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const types_1 = require("sonarwhal/dist/src/lib/types");
const misc_1 = require("sonarwhal/dist/src/lib/utils/misc");
const schema_validator_1 = require("sonarwhal/dist/src/lib/utils/schema-validator");
class BabelConfigParser extends types_1.Parser {
    constructor(sonarwhal) {
        super(sonarwhal);
        this.configFound = false;
        this.schema = misc_1.loadJSONFile(path.join(__dirname, 'schema', 'babelConfigSchema.json'));
        sonarwhal.on('fetch::end::json', this.parseBabelConfig.bind(this));
        sonarwhal.on('scan::end', this.parseEnd.bind(this));
    }
    async parseEnd(scanEnd) {
        if (!this.configFound) {
            await this.sonarwhal.emitAsync('parse::babel-config::error::not-found', scanEnd);
        }
    }
    async validateSchema(config, resource) {
        const validationResult = schema_validator_1.validate(this.schema, config);
        const valid = validationResult.valid;
        if (!valid) {
            const event = {
                errors: validationResult.errors,
                prettifiedErrors: validationResult.prettifiedErrors,
                resource
            };
            await this.sonarwhal.emitAsync('parse::babel-config::error::schema', event);
        }
        return validationResult;
    }
    async parseBabelConfig(fetchEnd) {
        const resource = fetchEnd.resource;
        const resourceFileName = path.basename(resource);
        const isPackageJson = resourceFileName === 'package.json';
        const isBabelrc = resourceFileName === '.babelrc';
        if (!isBabelrc && !isPackageJson) {
            return;
        }
        let config;
        try {
            const response = fetchEnd.response;
            const content = JSON.parse(response.body.content);
            if (isPackageJson && !content.babel) {
                return;
            }
            this.configFound = true;
            config = isPackageJson ? content.babel : content;
            const validationResult = await this.validateSchema(config, resource);
            if (!validationResult.valid) {
                return;
            }
            const event = {
                config: validationResult.data,
                originalConfig: config,
                resource
            };
            await this.sonarwhal.emitAsync('parse::babel-config::end', event);
        }
        catch (err) {
            const errorEvent = {
                error: err,
                resource
            };
            await this.sonarwhal.emitAsync('parse::babel-config::error::json', errorEvent);
        }
    }
}
exports.default = BabelConfigParser;
